/*
 * PSP Software Development Kit - http://www.pspdev.org
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PSPSDK root for details.
 *
 * psploadcore.h - Interface to LoadCoreForKernel.
 *
 * Copyright (c) 2005 Marcus R. Brown <mrbrown@ocgnet.org>
 * Copyright (c) 2005 James Forshaw <tyranid@gmail.com>
 * Copyright (c) 2005 John Kelley <ps2dev@kelley.ca>
 *
 * $Id: psploadcore.h 1095 2005-09-27 21:02:16Z jim $
 */

#ifndef PSPLOADCORE_H
#define PSPLOADCORE_H

#include <pspkerneltypes.h>

/** @defgroup LoadCore Interface to the LoadCoreForKernel library.
 */

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup LoadCore Interface to the LoadCoreForKernel library. */
/*@{*/

/** Describes a module.  This structure could change in future firmware revisions. */
// Complete structure by Davee
typedef struct SceModule2
{
	void* next; //0, 0x00
	u16 attribute; //4, 0x04
	u8 version[2]; //6, 0x06
	char modname[27]; //8, 0x08
	char terminal; //35, 0x23
	u16 status; //36, 0x24
	u16 unk_26; //38, 0x26
	u32 unk_28; //40, 0x28
	SceUID modid; //44, 0x2C
	SceUID usermod_thid; //48, 0x30
	SceUID memid; //52, 0x34
	SceUID mpidtext; //56, 0x38
	SceUID mpiddata; //60, 0x3C
	void *ent_top; //64, 0x40
	u32 ent_size; //68, 0x44
	void *stub_top; //72, 0x48
	u32 stub_size; //76, 0x4C
	int (* module_start)(SceSize, void *); //80, 0x50
	int (* module_stop)(SceSize, void *); //84, 0x54
	int (* module_bootstart)(SceSize, void *); //88, 0x58
	int (* module_reboot_before)(void *); //92, 0x5C
	int (* module_reboot_phase)(SceSize, void *); //96, 0x60
	u32 entry_addr; //100, 0x64
	u32 gp_value; //104, 0x68
	u32 text_addr; //108, 0x6C
	u32 text_size; //112, 0x70
	u32 data_size; //116, 0x74
	u32 bss_size; //120, 0x78
	u8 nsegment; //124, 0x7C
	u8 padding[3]; //125, 0x7D
	u32 segmentaddr[4]; //128, 0x80
	u32 segmentsize[4]; //144, 0x90
	int module_start_thread_priority; //160, 0xA0
	SceSize module_start_thread_stacksize; //164, 0xA4
	SceUInt module_start_thread_attr; //168, 0xA8
	int module_stop_thread_priority; //172, 0xAC
	SceSize module_stop_thread_stacksize; //176, 0xB0
	SceUInt module_stop_thread_attr; //180, 0xB4
	int module_reboot_before_thread_priority; //184, 0xB8
	SceSize module_reboot_before_thread_stacksize; //188, 0xBC
	SceUInt module_reboot_before_thread_attr; //192, 0xC0
} SceModule2;

/** Defines a library and its exported functions and variables.  Use the len
    member to determine the real size of the table (size = len * 4). */
typedef struct SceLibraryEntryTable {
	/**The library's name. */
	const char *		libname;
	/** Library version. */
	unsigned char		version[2];
	/** Library attributes. */
	unsigned short		attribute;
	/** Length of this entry table in 32-bit WORDs. */
	unsigned char		len;
	/** The number of variables exported by the library. */
	unsigned char		vstubcount;
	/** The number of functions exported by the library. */
	unsigned short		stubcount;
	/** Pointer to the entry table; an array of NIDs followed by
	    pointers to functions and variables. */
	void *				entrytable;
} SceLibraryEntryTable;

/** Specifies a library and a set of imports from that library.  Use the len
    member to determine the real size of the table (size = len * 4). */
typedef struct SceLibraryStubTable {
	/* The name of the library we're importing from. */
	const char *		libname;
	/** Minimum required version of the library we want to import. */
	unsigned char		version[2];
	/* Import attributes. */
	unsigned short		attribute;
	/** Length of this stub table in 32-bit WORDs. */
	unsigned char		len;
	/** The number of variables imported from the library. */
	unsigned char		vstubcount;
	/** The number of functions imported from the library. */
	unsigned short		stubcount;
	/** Pointer to an array of NIDs. */
	unsigned int *		nidtable;
	/** Pointer to the imported function stubs. */
	void *				stubtable;
	/** Pointer to the imported variable stubs. */
	void *				vstubtable;
} SceLibraryStubTable;

/**
 * Find a module by it's name.
 *
 * @param modname - The name of the module.
 *
 * @returns Pointer to the ::SceModule structure if found, otherwise NULL.
 */
SceModule2 *sceKernelFindModuleByName(const char *modname);

/**
 * Find a module from an address.
 *
 * @param addr - Address somewhere within the module.
 *
 * @returns Pointer to the ::SceModule structure if found, otherwise NULL.
 */
SceModule2 *sceKernelFindModuleByAddress(unsigned int addr);

/**
 * Find a module by it's UID.
 *
 * @param modid - The UID of the module.
 *
 * @returns Pointer to the ::SceModule structure if found, otherwise NULL.
 */
SceModule2 *sceKernelFindModuleByUID(SceUID modid);

/**
 * Return the count of loaded modules.
 *
 * @returns The count of loaded modules.
 */
int sceKernelModuleCount(void);

/**
 * Invalidate the CPU's instruction cache.
 */
void sceKernelIcacheClearAll(void);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* PSPLOADCORE_H */

